package monnethic.mobile.search;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.Toast;
import com.example.monnthic.monnethicmobile.R;

import java.util.ArrayList;

import monnethic.mobile.database.User;
import monnethic.mobile.homepage.InputController;
import monnethic.mobile.restApi.SessionApiHandler;
import monnethic.mobile.restApi.UserApiHandler;
import monnethic.mobile.restApi.WalletApiHandler;
import monnethic.mobile.wallet.Wallet;

public class SearchUser extends AppCompatActivity {
    EditText emailAddress;
    EditText phoneNumber;
    Button buttonCancel;
    Button buttonOk;
    private String session_id;
    private String user_hash;
    private String user_password;

    private String u_dest_n;
    private String u_dest_fn;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_search_user);

        Intent intent = getIntent();
        user_hash = intent.getStringExtra("USER_HASH");
        user_password = intent.getStringExtra("USER_PWD");
        session_id = intent.getStringExtra("SESSION_ID");

        initView();
    }

    private void initView(){
        emailAddress = findViewById(R.id.inputEmailSearch);
        phoneNumber = findViewById(R.id.inputPhoneSearch);
        buttonCancel = findViewById(R.id.buttonCancelSearch);
        buttonOk = findViewById(R.id.buttonOkSearch);

        buttonCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                setResult(2);
                finish();
            }
        });

        buttonOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                search();
            }
        });
    }

    private void search(){
        InputMethodManager inputMethodManager = (InputMethodManager) SearchUser.this.getSystemService(Activity.INPUT_METHOD_SERVICE);
        inputMethodManager.hideSoftInputFromWindow(SearchUser.this.getCurrentFocus().getWindowToken(),0);

        String uEmail = emailAddress.getText().toString();
        String uPhone = phoneNumber.getText().toString();

        if(InputController.isEmptyEdit(emailAddress) && InputController.isEmptyEdit(phoneNumber)){
            Toast.makeText(this, "Enter an email or a phone number", Toast.LENGTH_SHORT).show();
        } else if(InputController.isEmptyEdit(emailAddress)){
            new SearchUserTask().execute(null,uPhone);
        } else if(InputController.isEmptyEdit(phoneNumber)){
            if(!InputController.validEmail(emailAddress.getText().toString())){
                Toast.makeText(SearchUser.this, "Invalid format for email", Toast.LENGTH_SHORT).show();
            }else{
                new SearchUserTask().execute(uEmail,null);
            }
        } else {
            new SearchUserTask().execute(uEmail,uPhone);
        }
    }

    private class SearchUserTask extends AsyncTask<String,Void,ArrayList<Wallet>> {
        RelativeLayout progress = findViewById(R.id.progressLayout);
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            emailAddress.setEnabled(false);
            phoneNumber.setEnabled(false);
            buttonCancel.setEnabled(false);
            buttonOk.setEnabled(false);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.VISIBLE);
        }

        @Override
        protected void onPostExecute(ArrayList<Wallet> wallets) {
            try{
                if(wallets!=null && !wallets.isEmpty()){
                    ArrayList<DisplayWallet> displayWallets = new ArrayList<>();
                    for(Wallet w : wallets){
                        displayWallets.add(new DisplayWallet(u_dest_n,u_dest_fn,w.getWallet_hash(),w.getType()));
                    }
                    Intent displayWalletIntent = new Intent(SearchUser.this, DisplayWalletSearch.class);
                    displayWalletIntent.putExtra("data",displayWallets);
                    displayWalletIntent.putExtra("SESSION_ID",session_id);
                    displayWalletIntent.addFlags(Intent.FLAG_ACTIVITY_FORWARD_RESULT);
                    SearchUser.this.startActivity(displayWalletIntent);
                    finish();
                }else{
                    Toast.makeText(SearchUser.this,"User doesn't exist or don't have a wallet",Toast.LENGTH_SHORT).show();
                    emailAddress.setEnabled(true);
                    phoneNumber.setEnabled(true);
                    buttonCancel.setEnabled(true);
                    buttonOk.setEnabled(true);
                    getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                    progress.setVisibility(View.GONE);
                }
            }catch (Exception e){
                e.printStackTrace();
                emailAddress.setEnabled(true);
                phoneNumber.setEnabled(true);
                buttonCancel.setEnabled(true);
                buttonOk.setEnabled(true);
                getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                progress.setVisibility(View.GONE);
            }
        }

        @Override
        protected ArrayList<Wallet> doInBackground(String... strings) {
            WalletApiHandler walletApiHandler = new WalletApiHandler();
            ArrayList<Wallet> walletsList = new ArrayList<>();
            try {
                UserApiHandler userApiHandler = new UserApiHandler();
                User u;
                if(strings[0]!=null && strings[1]==null){
                    u = userApiHandler.getUser(1,strings[0],null);
                    u_dest_n = u.getName();
                    u_dest_fn = u.getFirstname();
                    walletsList = walletApiHandler.getUserWallets(new String[]{u.getUser_hash()});
                } else if(strings[0]==null && strings[1]!=null){
                    u = userApiHandler.getUser(2,null,strings[1]);
                    u_dest_n = u.getName();
                    u_dest_fn = u.getFirstname();
                    walletsList = walletApiHandler.getUserWallets(new String[]{u.getUser_hash()});
                } else {
                    u = userApiHandler.getUser(3,strings[0],strings[1]);
                    u_dest_n = u.getName();
                    u_dest_fn = u.getFirstname();
                    walletsList = walletApiHandler.getUserWallets(new String[]{u.getUser_hash()});
                }
            }catch (Exception e){
                e.printStackTrace();
            }
            return walletsList;
        }

    }

    @Override
    protected void onStop() {
        super.onStop();  // Always call the superclass method first
        new EndSessionTask().execute(session_id);
    }

    private class EndSessionTask extends AsyncTask<String,Void,Void> {
        @Override
        protected Void doInBackground(String... strings) {
            try{
                SessionApiHandler sessionApiHandler = new SessionApiHandler();
                sessionApiHandler.endSession(strings[0]);
            }catch (Exception e){
                e.printStackTrace();
            }
            return null;
        }
    }
}
