package monnethic.mobile.transaction;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.Toast;
import com.example.monnthic.monnethicmobile.R;

import monnethic.mobile.database.Transaction;
import monnethic.mobile.homepage.InputController;
import monnethic.mobile.restApi.SessionApiHandler;
import monnethic.mobile.restApi.TransactionApiHandler;
import monnethic.mobile.search.SearchUser;

public class TransactionActivity extends AppCompatActivity {
    private String wallet_hash;
    private String user_hash;
    private String user_password;
    private String session_id;
    private EditText addressDestination;
    private EditText amountEditText;
    private Button buttonCancel;
    private Button buttonOk;
    private Button buttonSearch;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_transaction);
        addressDestination = findViewById(R.id.destination_address);
        amountEditText = findViewById(R.id.amount);
        buttonCancel = findViewById(R.id.btn_cancel);
        buttonOk = findViewById(R.id.btn_send);
        buttonSearch = findViewById(R.id.buttonSearch);

        Intent intent = getIntent();
        user_hash = intent.getStringExtra("USER_HASH");
        user_password = intent.getStringExtra("USER_PWD");
        session_id = intent.getStringExtra("SESSION_ID");
        wallet_hash = intent.getStringExtra("WALLET_HASH");

        buttonCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                finish();
            }
        });

        buttonOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                InputMethodManager inputMethodManager = (InputMethodManager) TransactionActivity.this.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(TransactionActivity.this.getCurrentFocus().getWindowToken(),0);
                sendTransaction();
            }
        });

        buttonSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                InputMethodManager inputMethodManager = (InputMethodManager) TransactionActivity.this.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(TransactionActivity.this.getCurrentFocus().getWindowToken(),0);
                launchSearchActivity();
            }
        });

    }

    public void sendTransaction(){
        if(InputController.isEmptyEdit(addressDestination)){
            Toast.makeText(this, "No destination address", Toast.LENGTH_SHORT).show();
        } else if(InputController.isEmptyEdit(amountEditText)) {
            Toast.makeText(this, "No amount", Toast.LENGTH_SHORT).show();
        } else {
            String wallet_dest = addressDestination.getText().toString();
            String amount = amountEditText.getText().toString();
            try {
                new TransactionTask().execute(new SendingTransaction(user_hash,user_password,wallet_hash,wallet_dest,amount,"gonette"));
            } catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    public void launchSearchActivity(){
        Intent searchIntent = new Intent(TransactionActivity.this, SearchUser.class);
        searchIntent.putExtra("USER_HASH",user_hash);
        searchIntent.putExtra("USER_PWD",user_password);
        searchIntent.putExtra("SESSION_ID",session_id);
        startActivityForResult(searchIntent,1);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode==1){
            String walletHash = data.getStringExtra("walletHash");
            addressDestination.setText(walletHash);
        }
    }

    private class TransactionTask extends AsyncTask<SendingTransaction,String,String> {
        RelativeLayout progress = findViewById(R.id.progressLayout);

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            buttonCancel.setEnabled(false);
            buttonOk.setEnabled(false);
            buttonSearch.setEnabled(false);
            amountEditText.setEnabled(false);
            addressDestination.setEnabled(false);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.VISIBLE);
        }

        @Override
        protected String doInBackground(SendingTransaction... sendingTransactions) {
            try{
                TransactionApiHandler transactionApiHandler = new TransactionApiHandler();
                System.out.println("TRANSACTION: "+sendingTransactions[0].toString());
                return transactionApiHandler.doTransaction(sendingTransactions[0]);
            }catch (Exception e){
                e.printStackTrace();
            }
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            if(s!=null){
                Toast.makeText(TransactionActivity.this, "TxID : "+s, Toast.LENGTH_LONG).show();
            } else {
                Toast.makeText(TransactionActivity.this, "Couldn't get transaction id", Toast.LENGTH_SHORT).show();
            }
            finish();
        }
    }



    @Override
    protected void onStop() {
        super.onStop();  // Always call the superclass method first
        new EndSessionTask().execute(session_id);
    }

    private class EndSessionTask extends AsyncTask<String,Void,Void> {
        @Override
        protected Void doInBackground(String... strings) {
            try{
                SessionApiHandler sessionApiHandler = new SessionApiHandler();
                sessionApiHandler.endSession(strings[0]);
            }catch (Exception e){
                e.printStackTrace();
            }
            return null;
        }
    }

}
