package monnethic.mobile.wallet;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.example.monnthic.monnethicmobile.R;

import org.json.JSONObject;

import java.util.ArrayList;

import monnethic.mobile.database.User;
import monnethic.mobile.restApi.SessionApiHandler;
import monnethic.mobile.restApi.UserApiHandler;
import monnethic.mobile.restApi.WalletApiHandler;
import monnethic.mobile.settings.SettingsActivity;

public class HomeWalletActivity extends AppCompatActivity {
    private String user_hash;
    private String user_password;
    private String session_id;
    private String email;
    private Boolean approved;
    private Button buttonApproveUser;
    private Button buttonCreateWallet;
    private Button buttonSelectWallet;
    private Button buttonSettings;
    private RelativeLayout progress;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_home_wallet);
        buttonCreateWallet = findViewById(R.id.buttonCreateWallet);
        buttonSelectWallet = findViewById(R.id.buttonSelectWallet);
        buttonApproveUser = findViewById(R.id.buttonApproveUser);
        buttonSettings = findViewById(R.id.buttonSettings);


        Intent intent = getIntent();
        user_hash = intent.getStringExtra("USER_HASH");
        user_password = intent.getStringExtra("USER_PWD");
        System.out.println("USER_PWD: "+user_password);
        session_id = intent.getStringExtra("SESSION_ID");
        email = intent.getStringExtra("email");
        approved = Boolean.parseBoolean(intent.getStringExtra("approved"));


        buttonCreateWallet.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                launchCreateWalletActivity();
            }
        });
        buttonSettings.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                settings();
            }
        });
        buttonSelectWallet.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                launchSelectWalletActivity();
            }
        });

        if(approved){
            buttonApproveUser.setVisibility(View.INVISIBLE);
        }else{
            buttonApproveUser.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    approveUser();
                    approved = true;
                }
            });
        }
    }

    public void launchCreateWalletActivity() {
        try{
            if(approved){
                Intent createWalletIntent = new Intent(HomeWalletActivity.this, CreateWalletActivity.class);
                createWalletIntent.putExtra("USER_EMAIL",email);
                createWalletIntent.putExtra("USER_HASH",user_hash);
                createWalletIntent.putExtra("USER_PWD",user_password);
                createWalletIntent.putExtra("SESSION_ID",session_id);
                HomeWalletActivity.this.startActivity(createWalletIntent);
            }else{
                Toast.makeText(HomeWalletActivity.this,"You are not approved",Toast.LENGTH_SHORT).show();
            }
        } catch (Exception e){
            e.printStackTrace();
        }
    }

    public void launchSelectWalletActivity() {
        try{
            if(approved){
                String[] params = {user_hash};
                new getUserWalletTaskA().execute(params);
            }else {
                Toast.makeText(HomeWalletActivity.this,"You are not approved",Toast.LENGTH_SHORT).show();
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public void settings(){
        try{
            if(approved){
                String[] params = {user_hash};
                new getUserWalletTaskB().execute(params);
            }else {
                Toast.makeText(HomeWalletActivity.this,"You are not approved",Toast.LENGTH_SHORT).show();
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        super.onActivityResult(requestCode, resultCode, data);
        user_password = data.getStringExtra("new_password");
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
        System.out.println("USER_PWD: "+user_password);
        buttonCreateWallet.setEnabled(true);
        buttonSelectWallet.setEnabled(true);
        buttonSettings.setEnabled(true);
        buttonApproveUser.setEnabled(true);
    }

    public void approveUser(){
        new ApproveTask().execute(user_hash,email);
    }

    @Override
    protected void onResume() {
        super.onResume();
        buttonCreateWallet.setEnabled(true);
        buttonSelectWallet.setEnabled(true);
        buttonSettings.setEnabled(true);
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
    }


    //AsyncTask to get user wallets
    private class getUserWalletTaskA extends AsyncTask<String,ArrayList<Wallet>,ArrayList<Wallet>> {
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            progress = findViewById(R.id.progressLayoutSelect);
            buttonCreateWallet.setEnabled(false);
            buttonSelectWallet.setEnabled(false);
            buttonSettings.setEnabled(false);
            buttonApproveUser.setEnabled(false);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.VISIBLE);
        }

        @Override
        protected ArrayList<Wallet> doInBackground(String... params) {
            ArrayList<Wallet> walletsList = new ArrayList<>();
            try{
                WalletApiHandler walletApiHandler = new WalletApiHandler();
                walletsList = walletApiHandler.getUserWallets(params);
            }catch (Exception e){
                e.printStackTrace();
                getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                progress.setVisibility(View.GONE);
                buttonCreateWallet.setEnabled(true);
                buttonSelectWallet.setEnabled(true);
                buttonSettings.setEnabled(true);
                buttonApproveUser.setEnabled(true);
            }
            return walletsList;
        }

        @Override
        protected void onPostExecute(ArrayList<Wallet> listWallet) {
            if(listWallet!=null && !listWallet.isEmpty()){
                Intent selectWalletIntent = new Intent(HomeWalletActivity.this, SelectWalletActivity.class);
                selectWalletIntent.putExtra("USER_HASH",user_hash);
                selectWalletIntent.putExtra("USER_EMAIL",email);
                selectWalletIntent.putExtra("USER_PWD",user_password);
                selectWalletIntent.putExtra("SESSION_ID",session_id);
                selectWalletIntent.putExtra("data",listWallet);
                HomeWalletActivity.this.startActivity(selectWalletIntent);

                getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                progress.setVisibility(View.GONE);
                buttonCreateWallet.setEnabled(true);
                buttonSelectWallet.setEnabled(true);
                buttonSettings.setEnabled(true);
                buttonApproveUser.setEnabled(true);
            }else {
                Toast.makeText(HomeWalletActivity.this,"You don't have wallets",Toast.LENGTH_SHORT).show();
                getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                progress.setVisibility(View.GONE);
                buttonCreateWallet.setEnabled(true);
                buttonSelectWallet.setEnabled(true);
                buttonSettings.setEnabled(true);
                buttonApproveUser.setEnabled(true);
            }
        }
    }
    private class getUserWalletTaskB extends AsyncTask<String,ArrayList<Wallet>,ArrayList<Wallet>> {
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            progress = findViewById(R.id.progressLayoutSettings);
            buttonCreateWallet.setEnabled(false);
            buttonSelectWallet.setEnabled(false);
            buttonSettings.setEnabled(false);
            buttonApproveUser.setEnabled(false);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.VISIBLE);
        }

        @Override
        protected ArrayList<Wallet> doInBackground(String... params) {
            ArrayList<Wallet> walletsList = new ArrayList<>();
            try{
                WalletApiHandler walletApiHandler = new WalletApiHandler();
                walletsList = walletApiHandler.getUserWallets(params);
            }catch (Exception e){
                e.printStackTrace();
                getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                progress.setVisibility(View.GONE);
                buttonCreateWallet.setEnabled(true);
                buttonSelectWallet.setEnabled(true);
                buttonSettings.setEnabled(true);
                buttonApproveUser.setEnabled(true);
            }
            return walletsList;
        }

        @Override
        protected void onPostExecute(ArrayList<Wallet> listWallet) {
            Intent settingsIntent = new Intent(HomeWalletActivity.this, SettingsActivity.class);
            settingsIntent.putExtra("USER_HASH",user_hash);
            settingsIntent.putExtra("USER_EMAIL",email);
            settingsIntent.putExtra("USER_PWD",user_password);
            settingsIntent.putExtra("SESSION_ID",session_id);
            settingsIntent.putExtra("data",listWallet);
            HomeWalletActivity.this.startActivityForResult(settingsIntent,1);

            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.GONE);
            buttonCreateWallet.setEnabled(true);
            buttonSelectWallet.setEnabled(true);
            buttonSettings.setEnabled(true);
            buttonApproveUser.setEnabled(true);
        }
    }
    private class ApproveTask extends AsyncTask<String,String,String>{
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            progress = findViewById(R.id.progressLayoutSettings);
            buttonCreateWallet.setEnabled(false);
            buttonSelectWallet.setEnabled(false);
            buttonSettings.setEnabled(false);
            buttonApproveUser.setEnabled(false);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.VISIBLE);
        }

        @Override
        protected void onPostExecute(String result) {
            buttonApproveUser.setVisibility(View.INVISIBLE);
            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            progress.setVisibility(View.GONE);
            buttonCreateWallet.setEnabled(true);
            buttonSelectWallet.setEnabled(true);
            buttonSettings.setEnabled(true);
            buttonApproveUser.setEnabled(true);
        }

        @Override
        protected String doInBackground(String... strings) {
            try{
                UserApiHandler userApiHandler = new UserApiHandler();
                userApiHandler.approveUser(strings[0],strings[1]);
                return null;
            }catch (Exception e){
                e.printStackTrace();
                return null;
            }
        }
    }

    @Override
    protected void onStop() {
        super.onStop();  // Always call the superclass method first
        new EndSessionTask().execute(session_id);
    }

    private class EndSessionTask extends AsyncTask<String,Void,Void> {
        @Override
        protected Void doInBackground(String... strings) {
            try{
                SessionApiHandler sessionApiHandler = new SessionApiHandler();
                sessionApiHandler.endSession(strings[0]);
            }catch (Exception e){
                e.printStackTrace();
            }
            return null;
        }
    }

    /*
    private class CheckApprovalTask extends AsyncTask<String,Boolean,Boolean>{
        ProgressDialog progDailog = new ProgressDialog(HomeWalletActivity.this);
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            progDailog.setMessage("Loading...");
            progDailog.setIndeterminate(false);
            progDailog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            progDailog.setCancelable(true);
            progDailog.show();
        }

        @Override
        protected Boolean doInBackground(String... strings) {
            try{
                UserApiHandler userApiHandler = new UserApiHandler();
                return userApiHandler.checkApproval(strings[0],strings[1]);
            }catch (Exception e){
                return null;
            }
        }

        @Override
        protected void onPostExecute(Boolean result) {
            progDailog.dismiss();
        }
    }
    */


}