package monnethic.mobile.wallet;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import com.example.monnthic.monnethicmobile.R;
import java.util.ArrayList;
import monnethic.mobile.restApi.SessionApiHandler;
import monnethic.mobile.restApi.WalletApiHandler;
import monnethic.mobile.user.UserAccountActivity;

public class SelectWalletActivity extends AppCompatActivity {
    private ArrayList<Wallet> userWallets;
    private String user_hash;
    private String user_password;
    private String session_id;
    private ListView listView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_select_wallet);
        Intent intent = getIntent();
        user_hash = intent.getStringExtra("USER_HASH");
        user_password = intent.getStringExtra("USER_PWD");
        session_id = intent.getStringExtra("SESSION_ID");
        listView = (ListView) findViewById(R.id.listViewWallet);
        try{
            userWallets = (ArrayList<Wallet>) intent.getSerializableExtra("data");
        }catch (Exception e){
            e.printStackTrace();
            userWallets=null;
        }
        setupWallet();
    }

    public void setupWallet(){
        ListView listView = (ListView) findViewById(R.id.listViewWallet);
        WalletAdapter adapter = new WalletAdapter(this,userWallets);
        listView.setAdapter(adapter);

        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                Wallet w = (Wallet) listView.getItemAtPosition(i);
                launchAcountActivity(w);
            }
        });
    }

    private void updateWalletList(){
        new getUserWalletTask().execute(user_hash);
    }

    public void launchAcountActivity(Wallet w) {
        Intent accountIntent = new Intent(SelectWalletActivity.this, UserAccountActivity.class);
        accountIntent.putExtra("USER_HASH",user_hash);
        accountIntent.putExtra("USER_PWD",user_password);
        accountIntent.putExtra("SESSION_ID",session_id);
        accountIntent.putExtra("WALLET_HASH",w.getWallet_hash());
        accountIntent.putExtra("WALLET_BALANCE",String.valueOf(w.getBalance()));
        accountIntent.putExtra("WALLET_TYPE",w.getType());
        SelectWalletActivity.this.startActivity(accountIntent);
    }

    //AsyncTask to get user wallets
    private class getUserWalletTask extends AsyncTask<String,ArrayList<Wallet>,ArrayList<Wallet>> {
        @Override
        protected ArrayList<Wallet> doInBackground(String... params) {
            ArrayList<Wallet> walletsList = new ArrayList<>();
            try{
                WalletApiHandler walletApiHandler = new WalletApiHandler();
                walletsList = walletApiHandler.getUserWallets(params);
            }catch (Exception e){
                e.printStackTrace();
            }
            return walletsList;
        }

        @Override
        protected void onPostExecute(ArrayList<Wallet> listWallet) {
            if(listWallet!=null && !listWallet.isEmpty()){
                WalletAdapter adapter = new WalletAdapter(SelectWalletActivity.this,listWallet);
                listView.setAdapter(adapter);
                listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                        Wallet w = (Wallet) listView.getItemAtPosition(i);
                        launchAcountActivity(w);
                    }
                });
            }
        }
    }

    private class EndSessionTask extends AsyncTask<String,Void,Void> {
        @Override
        protected Void doInBackground(String... strings) {
            try{
                SessionApiHandler sessionApiHandler = new SessionApiHandler();
                sessionApiHandler.endSession(strings[0]);
            }catch (Exception e){
                e.printStackTrace();
            }
            return null;
        }
    }


    @Override
    protected void onStop() {
        super.onStop();  // Always call the superclass method first
        new EndSessionTask().execute(session_id);
    }

    @Override
    protected void onResume() {
        super.onResume();
        updateWalletList();
    }
}
